@@ -61,7 +61,7 @@ try:
 except ImportError:
     PyStringMap = None
 
-__all__ = ["Error","error","copy","deepcopy"]
+__all__ = ["Error", "error", "copy", "deepcopy"]
 
 def copy(x):
     """Shallow copy operation on arbitrary Python objects.
@@ -75,9 +75,15 @@ def copy(x):
         try:
             copier = x.__copy__
         except AttributeError:
-            raise error, \
-                  "un(shallow)copyable object of type %s" % type(x)
-        y = copier()
+            try:
+                reductor = x.__reduce__
+            except AttributeError:
+                raise error, \
+                      "un(shallow)copyable object of type %s" % type(x)
+            else:
+                y = _reconstruct(x, reductor(), 0)
+        else:
+            y = copier()
     else:
         y = copierfunction(x)
     return y
@@ -90,8 +96,15 @@ d[types.NoneType] = _copy_atomic
 d[types.IntType] = _copy_atomic
 d[types.LongType] = _copy_atomic
 d[types.FloatType] = _copy_atomic
+try:
+    d[types.ComplexType] = _copy_atomic
+except AttributeError:
+    pass
 d[types.StringType] = _copy_atomic
-d[types.UnicodeType] = _copy_atomic
+try:
+    d[types.UnicodeType] = _copy_atomic
+except AttributeError:
+    pass
 try:
     d[types.CodeType] = _copy_atomic
 except AttributeError:
@@ -153,9 +166,15 @@ def deepcopy(x, memo = None):
         try:
             copier = x.__deepcopy__
         except AttributeError:
-            raise error, \
-                  "un-deep-copyable object of type %s" % type(x)
-        y = copier(memo)
+            try:
+                reductor = x.__reduce__
+            except AttributeError:
+                raise error, \
+                      "un-deep-copyable object of type %s" % type(x)
+            else:
+                y = _reconstruct(x, reductor(), 1)
+        else:
+            y = copier(memo)
     else:
         y = copierfunction(x, memo)
     memo[d] = y
@@ -169,8 +188,15 @@ d[types.NoneType] = _deepcopy_atomic
 d[types.IntType] = _deepcopy_atomic
 d[types.LongType] = _deepcopy_atomic
 d[types.FloatType] = _deepcopy_atomic
+try:
+    d[types.ComplexType] = _deepcopy_atomic
+except AttributeError:
+    pass
 d[types.StringType] = _deepcopy_atomic
-d[types.UnicodeType] = _deepcopy_atomic
+try:
+    d[types.UnicodeType] = _deepcopy_atomic
+except AttributeError:
+    pass
 d[types.CodeType] = _deepcopy_atomic
 d[types.TypeType] = _deepcopy_atomic
 d[types.XRangeType] = _deepcopy_atomic
@@ -253,6 +279,26 @@ def _deepcopy_inst(x, memo):
     return y
 d[types.InstanceType] = _deepcopy_inst
 
+def _reconstruct(x, info, deep):
+    if isinstance(info, str):
+        return x
+    assert isinstance(info, tuple)
+    n = len(info)
+    assert n in (2, 3)
+    callable, args = info[:2]
+    if n > 2:
+        state = info[2]
+    else:
+        state = {}
+    if deep:
+        args = deepcopy(args)
+    y = callable(*args)
+    if state:
+        if deep:
+            state = deepcopy(state)
+        y.__dict__.update(state)
+    return y
+
 del d
 
 del types
