@@ -25,19 +25,19 @@ request. This diagram details these state transitions:
       v
     Unread-response   [Response-headers-read]
       |\____________________
-      |                     \
-      | response.read()      | putrequest()
-      v                      v
-    Idle                   Req-started-unread-response
-                     _______/|
-                    /        |
-   response.read() |         | ( putheader() )*  endheaders()
-                   v         v
-       Request-started     Req-sent-unread-response
-                             |
-                             | response.read()
-                             v
-                           Request-sent
+      |                     |
+      | response.read()     | putrequest()
+      v                     v
+    Idle                  Req-started-unread-response
+                     ______/|
+                   /        |
+   response.read() |        | ( putheader() )*  endheaders()
+                   v        v
+       Request-started    Req-sent-unread-response
+                            |
+                            | response.read()
+                            v
+                          Request-sent
 
 This diagram presents the following rules:
   -- a second request may not be started until {response-headers-read}
@@ -66,15 +66,22 @@ Req-started-unread-response    _CS_REQ_STARTED    <response_class>
 Req-sent-unread-response       _CS_REQ_SENT       <response_class>
 """
 
-import socket
-import string
+import errno
 import mimetools
+import socket
 
 try:
     from cStringIO import StringIO
 except ImportError:
     from StringIO import StringIO
 
+__all__ = ["HTTP", "HTTPResponse", "HTTPConnection", "HTTPSConnection",
+           "HTTPException", "NotConnected", "UnknownProtocol",
+           "UnknownTransferEncoding", "IllegalKeywordArgument",
+           "UnimplementedFileMode", "IncompleteRead",
+           "ImproperConnectionState", "CannotSendRequest", "CannotSendHeader",
+           "ResponseNotReady", "BadStatusLine", "error"]
+
 HTTP_PORT = 80
 HTTPS_PORT = 443
 
@@ -94,14 +101,14 @@ class HTTPResponse:
         self.msg = None
 
         # from the Status-Line of the response
-        self.version = _UNKNOWN	# HTTP-Version
-        self.status = _UNKNOWN	# Status-Code
-        self.reason = _UNKNOWN	# Reason-Phrase
+        self.version = _UNKNOWN # HTTP-Version
+        self.status = _UNKNOWN  # Status-Code
+        self.reason = _UNKNOWN  # Reason-Phrase
 
-        self.chunked = _UNKNOWN		# is "chunked" being used?
-        self.chunk_left = _UNKNOWN	# bytes left to read in current chunk
-        self.length = _UNKNOWN		# number of bytes left in response
-        self.will_close = _UNKNOWN	# conn will close at end of response
+        self.chunked = _UNKNOWN         # is "chunked" being used?
+        self.chunk_left = _UNKNOWN      # bytes left to read in current chunk
+        self.length = _UNKNOWN          # number of bytes left in response
+        self.will_close = _UNKNOWN      # conn will close at end of response
 
     def begin(self):
         if self.msg is not None:
@@ -112,10 +119,10 @@ class HTTPResponse:
         if self.debuglevel > 0:
             print "reply:", repr(line)
         try:
-            [version, status, reason] = string.split(line, None, 2)
+            [version, status, reason] = line.split(None, 2)
         except ValueError:
             try:
-                [version, status] = string.split(line, None, 1)
+                [version, status] = line.split(None, 1)
                 reason = ""
             except ValueError:
                 version = "HTTP/0.9"
@@ -125,13 +132,19 @@ class HTTPResponse:
             self.close()
             raise BadStatusLine(line)
 
-        self.status = status = int(status)
-        self.reason = string.strip(reason)
+        # The status code is a three-digit number
+        try:
+            self.status = status = int(status)
+            if status < 100 or status > 999:
+                raise BadStatusLine(line)
+        except ValueError:
+            raise BadStatusLine(line)
+        self.reason = reason.strip()
 
         if version == 'HTTP/1.0':
             self.version = 10
         elif version.startswith('HTTP/1.'):
-            self.version = 11	# use HTTP/1.1 code for HTTP/1.x where x>=1
+            self.version = 11   # use HTTP/1.1 code for HTTP/1.x where x>=1
         elif version == 'HTTP/0.9':
             self.version = 9
         else:
@@ -152,7 +165,7 @@ class HTTPResponse:
         # are we using the chunked-style of transfer encoding?
         tr_enc = self.msg.getheader('transfer-encoding')
         if tr_enc:
-            if string.lower(tr_enc) != 'chunked':
+            if tr_enc.lower() != 'chunked':
                 raise UnknownTransferEncoding()
             self.chunked = 1
             self.chunk_left = None
@@ -162,11 +175,11 @@ class HTTPResponse:
         # will the connection close at the end of the response?
         conn = self.msg.getheader('connection')
         if conn:
-            conn = string.lower(conn)
+            conn = conn.lower()
             # a "Connection: close" will always close the connection. if we
             # don't see that and this is not HTTP/1.1, then the connection will
             # close unless we see a Keep-Alive header.
-            self.will_close = string.find(conn, 'close') != -1 or \
+            self.will_close = conn.find('close') != -1 or \
                               ( self.version != 11 and \
                                 not self.msg.getheader('keep-alive') )
         else:
@@ -187,9 +200,9 @@ class HTTPResponse:
             self.length = None
 
         # does the body have a fixed length? (of zero)
-        if (status == 204 or		# No Content
-            status == 304 or		# Not Modified
-            100 <= status < 200):	# 1xx codes
+        if (status == 204 or            # No Content
+            status == 304 or            # Not Modified
+            100 <= status < 200):       # 1xx codes
             self.length = 0
 
         # if the connection remains open, and we aren't using chunked, and
@@ -224,10 +237,10 @@ class HTTPResponse:
             while 1:
                 if chunk_left is None:
                     line = self.fp.readline()
-                    i = string.find(line, ';')
+                    i = line.find(';')
                     if i >= 0:
-                        line = line[:i]	# strip chunk-extensions
-                    chunk_left = string.atoi(line, 16)
+                        line = line[:i] # strip chunk-extensions
+                    chunk_left = int(line, 16)
                     if chunk_left == 0:
                         break
                 if amt is None:
@@ -238,7 +251,7 @@ class HTTPResponse:
                     return value
                 elif amt == chunk_left:
                     value = value + self._safe_read(amt)
-                    self._safe_read(2)	# toss the CRLF at the end of the chunk
+                    self._safe_read(2)  # toss the CRLF at the end of the chunk
                     self.chunk_left = None
                     return value
                 else:
@@ -246,7 +259,7 @@ class HTTPResponse:
                     amt = amt - chunk_left
 
                 # we read the whole chunk, get another
-                self._safe_read(2)	# toss the CRLF at the end of the chunk
+                self._safe_read(2)      # toss the CRLF at the end of the chunk
                 chunk_left = None
 
             # read and discard trailer up to the CRLF terminator
@@ -267,7 +280,7 @@ class HTTPResponse:
                 s = self.fp.read()
             else:
                 s = self._safe_read(self.length)
-            self.close()	# we read everything
+            self.close()        # we read everything
             return s
 
         if self.length is not None:
@@ -331,7 +344,7 @@ class HTTPConnection:
 
     def _set_hostport(self, host, port):
         if port is None:
-            i = string.find(host, ':')
+            i = host.find(':')
             if i >= 0:
                 port = int(host[i+1:])
                 host = host[:i]
@@ -345,15 +358,29 @@ class HTTPConnection:
 
     def connect(self):
         """Connect to the host and port specified in __init__."""
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        if self.debuglevel > 0:
-            print "connect: (%s, %s)" % (self.host, self.port)
-        self.sock.connect((self.host, self.port))
+        msg = "getaddrinfo returns an empty list"
+        for res in socket.getaddrinfo(self.host, self.port, 0, socket.SOCK_STREAM):
+            af, socktype, proto, canonname, sa = res
+            try:
+                self.sock = socket.socket(af, socktype, proto)
+                if self.debuglevel > 0:
+                    print "connect: (%s, %s)" % (self.host, self.port)
+                self.sock.connect(sa)
+            except socket.error, msg:
+                if self.debuglevel > 0:
+                    print 'connect fail:', (self.host, self.port)
+                if self.sock:
+                    self.sock.close()
+                self.sock = None
+                continue
+            break
+        if not self.sock:
+            raise socket.error, msg
 
     def close(self):
         """Close the connection to the HTTP server."""
         if self.sock:
-            self.sock.close()	# close it manually... there may be other refs
+            self.sock.close()   # close it manually... there may be other refs
             self.sock = None
         if self.__response:
             self.__response.close()
@@ -378,7 +405,7 @@ class HTTPConnection:
         try:
             self.sock.send(str)
         except socket.error, v:
-            if v[0] == 32:	# Broken pipe
+            if v[0] == 32:      # Broken pipe
                 self.close()
             raise
 
@@ -439,7 +466,12 @@ class HTTPConnection:
             # be using HTTP/1.0 and those clients may be issuing this header
             # themselves. we should NOT issue it twice; some web servers (such
             # as Apache) barf when they see two Host: headers
-            self.putheader('Host', self.host)
+
+            # if we need a non-standard port,include it in the header
+            if self.port == HTTP_PORT:
+                self.putheader('Host', self.host)
+            else:
+                self.putheader('Host', "%s:%s" % (self.host, self.port))
 
             # note: we are assuming that clients will not attempt to set these
             #       headers since *this* library must deal with the
@@ -564,18 +596,31 @@ class FakeSocket:
         interface of a real socket.  It only supports modes 'r' and
         'rb' and the bufsize argument is ignored.
 
-        The returned object contains *all* of the file data 
+        The returned object contains *all* of the file data
         """
         if mode != 'r' and mode != 'rb':
             raise UnimplementedFileMode()
 
-        msgbuf = ""
+        msgbuf = []
         while 1:
             try:
-                msgbuf = msgbuf + self.__ssl.read()
-            except socket.sslerror, msg:
+                buf = self.__ssl.read()
+            except socket.sslerror, err:
+                if (err[0] == socket.SSL_ERROR_WANT_READ
+                    or err[0] == socket.SSL_ERROR_WANT_WRITE
+                    or 0):
+                    continue
+                if err[0] == socket.SSL_ERROR_ZERO_RETURN:
+                    break
+                raise
+            except socket.error, err:
+                if err[0] == errno.EINTR:
+                    continue
+                raise
+            if buf == '':
                 break
-        return StringIO(msgbuf)
+            msgbuf.append(buf)
+        return StringIO("".join(msgbuf))
 
     def send(self, stuff, flags = 0):
         return self.__ssl.write(stuff)
@@ -613,7 +658,10 @@ class HTTPSConnection(HTTPConnection):
 
         sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
         sock.connect((self.host, self.port))
-        ssl = socket.ssl(sock, self.key_file, self.cert_file)
+        realsock = sock
+        if hasattr(sock, "_sock"):
+            realsock = sock._sock
+        ssl = socket.ssl(realsock, self.key_file, self.cert_file)
         self.sock = FakeSocket(sock, ssl)
 
 
@@ -627,7 +675,7 @@ class HTTP:
 
     _connection_class = HTTPConnection
 
-    def __init__(self, host='', port=None, **x509):
+    def __init__(self, host='', port=None):
         "Provide a default host, since the superclass requires one."
 
         # some joker passed 0 explicitly, meaning default port
@@ -637,18 +685,19 @@ class HTTP:
         # Note that we may pass an empty string as the host; this will throw
         # an error when we attempt to connect. Presumably, the client code
         # will call connect before then, with a proper host.
-        self._conn = self._connection_class(host, port)
-        # set up delegation to flesh out interface
-        self.send = self._conn.send
-        self.putrequest = self._conn.putrequest
-        self.endheaders = self._conn.endheaders
-        self._conn._http_vsn = self._http_vsn
-        self._conn._http_vsn_str = self._http_vsn_str
+        self._setup(self._connection_class(host, port))
 
-        # we never actually use these for anything, but we keep them here for
-        # compatibility with post-1.5.2 CVS.
-        self.key_file = x509.get('key_file')
-        self.cert_file = x509.get('cert_file')
+    def _setup(self, conn):
+        self._conn = conn
+
+        # set up delegation to flesh out interface
+        self.send = conn.send
+        self.putrequest = conn.putrequest
+        self.endheaders = conn.endheaders
+        self.set_debuglevel = conn.set_debuglevel
+
+        conn._http_vsn = self._http_vsn
+        conn._http_vsn_str = self._http_vsn_str
 
         self.file = None
 
@@ -659,17 +708,13 @@ class HTTP:
             self._conn._set_hostport(host, port)
         self._conn.connect()
 
-    def set_debuglevel(self, debuglevel):
-        self._conn.set_debuglevel(debuglevel)
-
     def getfile(self):
         "Provide a getfile, since the superclass' does not use this concept."
         return self.file
 
     def putheader(self, header, *values):
         "The superclass allows only one value argument."
-        self._conn.putheader(header,
-                             string.joinfields(values, '\r\n\t'))
+        self._conn.putheader(header, '\r\n\t'.join(values))
 
     def getreply(self):
         """Compat definition since superclass does not define it.
@@ -715,11 +760,24 @@ if hasattr(socket, 'ssl'):
 
         Python 1.5.2 did not have an HTTPS class, but it defined an
         interface for sending http requests that is also useful for
-        https. 
+        https.
         """
 
         _connection_class = HTTPSConnection
-        
+
+        def __init__(self, host='', port=None, **x509):
+            # provide a default host, pass the X509 cert info
+
+            # urf. compensate for bad input.
+            if port == 0:
+                port = None
+            self._setup(self._connection_class(host, port, **x509))
+
+            # we never actually use these for anything, but we keep them
+            # here for compatibility with post-1.5.2 CVS.
+            self.key_file = x509.get('key_file')
+            self.cert_file = x509.get('cert_file')
+
 
 class HTTPException(Exception):
     pass
@@ -795,12 +853,13 @@ def test():
     print 'reason =', reason
     print
     if headers:
-        for header in headers.headers: print string.strip(header)
+        for header in headers.headers: print header.strip()
     print
     print h.getfile().read()
 
     if hasattr(socket, 'ssl'):
         host = 'sourceforge.net'
+        selector = '/projects/python'
         hs = HTTPS()
         hs.connect(host)
         hs.putrequest('GET', selector)
@@ -810,7 +869,7 @@ def test():
         print 'reason =', reason
         print
         if headers:
-            for header in headers.headers: print string.strip(header)
+            for header in headers.headers: print header.strip()
         print
         print hs.getfile().read()
 
