@@ -13,6 +13,30 @@ tempdir = None
 template = None
 
 def gettempdir():
+    """Function to calculate the directory to use."""
+    global tempdir
+    if tempdir is not None:
+        return tempdir
+
+    # _gettempdir_inner deduces whether a candidate temp dir is usable by
+    # trying to create a file in it, and write to it.  If that succeeds,
+    # great, it closes the file and unlinks it.  There's a race, though:
+    # the *name* of the test file it tries is the same across all threads
+    # under most OSes (Linux is an exception), and letting multiple threads
+    # all try to open, write to, close, and unlink a single file can cause
+    # a variety of bogus errors (e.g., you cannot unlink a file under
+    # Windows if anyone has it open, and two threads cannot create the
+    # same file in O_EXCL mode under Unix).  The simplest cure is to serialize
+    # calls to _gettempdir_inner.  This isn't a real expense, because the
+    # first thread to succeed sets the global tempdir, and all subsequent
+    # calls to gettempdir() reuse that without trying _gettempdir_inner.
+    _tempdir_lock.acquire()
+    try:
+        return _gettempdir_inner()
+    finally:
+        _tempdir_lock.release()
+
+def _gettempdir_inner():
     """Function to calculate the directory to use."""
     global tempdir
     if tempdir is not None:
@@ -37,6 +61,10 @@ def gettempdir():
             attempdirs.insert(0, dirname)
         except macfs.error:
             pass
+    elif os.name == 'riscos':
+        scrapdir = os.getenv('Wimp$ScrapDir')
+        if scrapdir:
+            attempdirs.insert(0, scrapdir)
     for envname in 'TMPDIR', 'TEMP', 'TMP':
         if os.environ.has_key(envname):
             attempdirs.insert(0, os.environ[envname])
@@ -90,7 +118,7 @@ if os.name == "posix":
 # string.
 elif os.name == "nt":
     template = '~' + `os.getpid()` + '-'
-elif os.name == 'mac':
+elif os.name in ('mac', 'riscos'):
     template = 'Python-Tmp-'
 else:
     template = 'tmp' # XXX might choose a better one
@@ -127,17 +155,26 @@ class TemporaryFileWrapper:
     In particular, it seeks to automatically remove the file when it is
     no longer needed.
     """
+
+    # Cache the unlinker so we don't get spurious errors at shutdown
+    # when the module-level "os" is None'd out.  Note that this must
+    # be referenced as self.unlink, because the name TemporaryFileWrapper
+    # may also get None'd out before __del__ is called.
+    unlink = os.unlink
+
     def __init__(self, file, path):
         self.file = file
         self.path = path
+        self.close_called = 0
 
     def close(self):
-        self.file.close()
-        os.unlink(self.path)
+        if not self.close_called:
+            self.close_called = 1
+            self.file.close()
+            self.unlink(self.path)
 
     def __del__(self):
-        try: self.close()
-        except: pass
+        self.close()
 
     def __getattr__(self, name):
         file = self.__dict__['file']
@@ -169,8 +206,8 @@ def TemporaryFile(mode='w+b', bufsize=-1, suffix=""):
 # multiple threads will never see the same integer).  The integer will
 # usually be a Python int, but if _counter.get_next() is called often
 # enough, it will become a Python long.
-# Note that the only name that survives this next block of code
-# is "_counter".
+# Note that the only names that survive this next block of code
+# are "_counter" and "_tempdir_lock".
 
 class _ThreadSafeCounter:
     def __init__(self, mutex, initialvalue=0):
@@ -199,10 +236,12 @@ except ImportError:
         release = acquire
 
     _counter = _ThreadSafeCounter(_DummyMutex())
+    _tempdir_lock = _DummyMutex()
     del _DummyMutex
 
 else:
     _counter = _ThreadSafeCounter(thread.allocate_lock())
+    _tempdir_lock = thread.allocate_lock()
     del thread
 
 del _ThreadSafeCounter
